/*
 *	linux/fs/rbf/rbf_defs.h
 *	RBF File System - private definitions
 *	
 *	Copyright (C) 2001 Andrew Cannon <ajc@gmx.net>
 * 	
 *  This file is subject to the terms and conditions of the
 *  GNU General Public License. See the file COPYING in the
 *  main directory of this archive for more details.
 */

#ifndef __RBF_DEFS_H_INCLUDED__
#define __RBF_DEFS_H_INCLUDED__

#define NODEBUG
//#undef NODEBUG

#ifndef NODEBUG
#define DPRINTK(format, args...) printk(KERN_WARNING "%s: ", __FUNCTION__); printk(KERN_WARNING format, ## args)
#else
#define DPRINTK(x...)
#endif

#define RBF_VERSION	0
#define RBF_REVISION	63
#define RBF_LEVEL	"(alpha)"

#include <linux/module.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/version.h>
#include <linux/rbf_fs.h>
#include <linux/rbf_fs_sb.h>
#include <linux/rbf_fs_i.h>
#include <linux/smp_lock.h>
#include <linux/pagemap.h>
#include <linux/buffer_head.h>
#include <linux/vfs.h>
#include <linux/sched.h>
#include <linux/pagemap.h>

#include <asm/byteorder.h>
#include <asm/uaccess.h>

/*
 *	superblock/inode private data fields
 */
#define s_su_sbh		(rbf_sb(s))->su_sbh
#define s_su_rbf_sb		rbf_sb(s)->su_rbf_sb
#define s_su_operations		rbf_sb(s)->su_operations
#define s_su_minalloc		rbf_sb(s)->su_minalloc
#define s_su_umask		rbf_sb(s)->su_umask
#define s_su_debug		rbf_sb(s)->su_debug
#define s_su_quiet		rbf_sb(s)->su_quiet
#define s_su_verbose		rbf_sb(s)->su_verbose
#define s_su_dealloc		rbf_sb(s)->su_dealloc
#define s_su_keepcase		rbf_sb(s)->su_keepcase

#define s_su_paranoid		rbf_sb(s)->su_paranoid
#define s_su_nomagic		rbf_sb(s)->su_nomagic
#define s_su_nonamecheck	rbf_sb(s)->su_nonamecheck
#define s_su_minalloc_k		rbf_sb(s)->su_minalloc_k
#define s_su_hardlinks		rbf_sb(s)->su_hardlinks
#define s_su_owner		rbf_sb(s)->su_owner

#define s_su_uid		rbf_sb(s)->su_uid
#define s_su_gid		rbf_sb(s)->su_gid
#define s_su_sectsize		rbf_sb(s)->su_sectsize
#define s_su_sectsize_bits	rbf_sb(s)->su_sectsize_bits
#define s_su_sect_shift		rbf_sb(s)->su_sect_shift
#define s_su_sect_mask		rbf_sb(s)->su_sect_mask
#define s_su_part_shift		rbf_sb(s)->su_part_shift
#define s_su_part_mask		rbf_sb(s)->su_part_mask
#define s_su_nsect		rbf_sb(s)->su_nsect
#define s_su_freesect		rbf_sb(s)->su_freesect
#define s_su_maxseg		rbf_sb(s)->su_maxseg
#define s_su_mapblk		rbf_sb(s)->su_mapblk
#define s_su_mapoffs		rbf_sb(s)->su_mapoffs
#define s_su_curmap		rbf_sb(s)->su_curmap
#define s_su_clus		rbf_sb(s)->su_clus
#define s_su_clus_bits		rbf_sb(s)->su_clus_bits
#define s_su_nclus		rbf_sb(s)->su_nclus
#define s_su_root		rbf_sb(s)->su_root
#define s_su_setuid		rbf_sb(s)->su_setuid
#define s_su_setgid		rbf_sb(s)->su_setgid

#define i_mmu_private		rbf_i(inode)->mmu_private
#define i_iu_segnum		rbf_i(inode)->iu_segnum
#define i_iu_segaddr		rbf_i(inode)->iu_segaddr
#define i_iu_segstart		rbf_i(inode)->iu_segstart
#define i_iu_segend		rbf_i(inode)->iu_segend
#define i_iu_nsect		rbf_i(inode)->iu_nsect
#define i_iu_create		rbf_i(inode)->iu_create

struct rbf_operations {
	struct inode_operations		*dir_inode_ops;
	struct file_operations		*dir_operations;
	struct inode_operations		*file_inode_ops;
	struct file_operations		*file_operations;
	struct address_space_operations	*address_space_ops;
};



time_t rbf_rdtime(const u8 *p, int n);
void rbf_wrtime(u8 *p, int n, time_t t);
int rbf_readname(char *buf, const char *s);
int rbf_validate_name(struct super_block *s, const char *name, int namelen);

int rbf_extend_allocation(struct super_block *s, sector_t sector, int nsect);
int rbf_allocate(struct super_block *s, sector_t *sectorp, int nsect);
void rbf_deallocate(struct super_block *s, sector_t sector, int nsect);
long rbf_free_blocks(struct super_block *s);

int rbf_get_block(struct inode *inode, sector_t block, struct buffer_head *bh_result, int create);
long rbf_find_sector(struct inode *inode, sector_t sector, int *lengthp, int create);
int rbf_contract(struct inode *inode, sector_t sector);
long rbf_scan_fd(struct inode *inode, struct rbf_inode *ip);
struct inode *rbf_new_inode(const struct inode *dir, int mode);
int rbf_sync_inode(struct inode *inode);
int rbf_release_file(struct inode *inode, struct file *filp);



/*
 * NOTE! unlike strncmp, rbf_match returns 1 for success, 0 for failure.
 *
 * len <= RBF_NAMELEN and de != NULL are guaranteed by caller.
 */
static inline int rbf_match(struct super_block *s, int len, const char *name, const char *buffer)
{
	/*
	 *	Original RBF only set b7 of last char (no null termination)
	 *	and allowed names of up to 29 chars.
	 *	Later versions null terminate _and_ set bit 7!!
	 *	We don't support 29 char names (truncate/ignore) but we
	 *	do allow either form of termination.
	 */
	int lm1 = len - 1;
	int fold = !s_su_keepcase;
	
	if ((fold ? strnicmp : strncmp)(name, buffer, lm1) != 0)
		return 0;
	if ((buffer[lm1] & 0x80) && ((buffer[lm1] ^ name[lm1]) & (fold ? 0x5f : 0x7f)) == 0)
		return 1;
	if (((buffer[lm1] ^ name[lm1]) & (fold ? 0xdf : 0xff)) == 0 && (len == RBF_NAMELEN || buffer[len] == 0 ))
		return 1;
	return 0;
}

/*
 *	convert inode# to sector#
 */
static inline long inode_adrs(struct super_block *s, ino_t ino)
{
	return (long)ino;
}

/*
 *	convert inode# to block#
 *	NB: only valid if sectsize >= blocksize
 */
static inline long inode_block(struct super_block *s, ino_t ino)
{
	return inode_adrs(s, ino) << s_su_sect_shift;
}

/*
 *	convert sectors to 512 byte blocks for inode::i_blocks (round up)
 */
static inline long sect_iblock(struct super_block *s, sector_t sectors)
{
	return ((sectors + s_su_part_mask) << (s_su_sectsize_bits - 8)) >> 1;
}

/*
 *	read block containing sector & return offset to sector
 */
static inline struct buffer_head *rbf_bread(struct super_block *s, long psn, int *offsetp)
{
	*offsetp = ((psn & s_su_part_mask) << s_su_sectsize_bits);
	return sb_bread(s, (psn << s_su_sect_shift) >> s_su_part_shift);
}

/*
 *	read buffer at logical file position
 */
static inline struct buffer_head *rbf_logical_bread(struct inode *inode, unsigned long pos, int *offsetp)
{
	struct super_block *s = inode->i_sb;
	struct buffer_head *bh = 0;
	int offset = 0;
	int psn = rbf_find_sector(inode, pos >> s_su_sectsize_bits, 0, 0);
	
	if (psn)
		bh = rbf_bread(s, psn, &offset);
	*offsetp = offset + (pos & (s_su_sectsize - 1));
	return bh;
}


/*
 *	update buffer
 */
static inline void rbf_update_buffer(int issync, struct buffer_head *bh)
{
	mark_buffer_dirty(bh);
	if (issync) {
		ll_rw_block(WRITE, 1, &bh);
		wait_on_buffer(bh);
	}
}

extern struct inode_operations rbf_file_inode_ops;

extern struct inode_operations rbf_page_dir_inode_ops;
extern struct file_operations rbf_page_dir_operations;
extern struct file_operations rbf_page_file_operations;
extern struct address_space_operations rbf_page_address_space_ops;

extern struct inode_operations rbf_block_dir_inode_ops;
extern struct file_operations rbf_block_dir_operations;
extern struct file_operations rbf_block_file_operations;
extern struct address_space_operations rbf_block_address_space_ops;

#endif /* __RBF_DEFS_H_INCLUDED__ */

